﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class RabboniModule : MonoBehaviour
{
	public RabboniConsole console;
	
	private string targetAddress;
	public Text targetAddressText;
	public string targetAddressKey = "Module";
	
	public Text statusText;
	public GameObject buttonSet;
	public GameObject buttonStopScan;
	
	public bool subscribeChecked;
	public bool prefsChecked;
	public bool isConnected;
	
	public Vector3 lastAcc;
	public Vector3 lastGyro;
	
    // Start is called before the first frame update
    void Start()
    {
		Initialize();
    }
	
	public void Initialize()
	{
		targetAddress = PlayerPrefs.GetString(targetAddressKey, "(請掃描Rabboni裝置)");
		targetAddressText.text = System.String.Format("目標裝置 MAC: {0}", targetAddress);
		
		statusText.text = "狀態: 等待指令";
				
	}
    public void Scan()
	{
		statusText.text = "狀態: 搜尋Rabboni裝置...";
		targetAddressText.text = "目標裝置 MAC: ----";
		
		buttonSet.SetActive(false);
		buttonStopScan.SetActive(true);

		BluetoothLEHardwareInterface.ScanForPeripheralsWithServices (null, (address, name) => {

			// we only want to look at devices that have the name we are looking for
			// this is the best way to filter out devices
			for(int i = 0; i < console.DeviceName.Length; i++)
			{
				if (name.Contains (console.DeviceName[i]))
				{
					// it is always a good idea to stop scanning while you connect to a device
					// and get things set up
					BluetoothLEHardwareInterface.StopScan ();

					targetAddress = address;

					statusText.text = "狀態: 成功找到Rabboni";
					
					targetAddressText.text = System.String.Format("目標裝置 MAC: {0}", targetAddress);
					PlayerPrefs.SetString(targetAddressKey, targetAddress);
					buttonSet.SetActive(true);
					buttonStopScan.SetActive(false);
					break;
				}
			}
		}, null, false, false);
	}
	
	public void StopScan()
	{
		BluetoothLEHardwareInterface.StopScan ();
		statusText.text = "狀態: 已停止掃描";	
		buttonSet.SetActive(true);
		buttonStopScan.SetActive(false);
	}
	
	
	public void ScanAndConnect()
	{
		statusText.text = System.String.Format("狀態: 尋找MAC: {0}", targetAddress);
		
		buttonSet.SetActive(false);
		buttonStopScan.SetActive(true);

		BluetoothLEHardwareInterface.ScanForPeripheralsWithServices (null, (address, name) => {

			
			if (address == targetAddress)
			{
				
				BluetoothLEHardwareInterface.StopScan ();
				statusText.text = System.String.Format("狀態: 成功找到MAC: {0}", targetAddress);
				
				buttonStopScan.SetActive(false);
				
				Invoke("Connect", 0.5f);
			}

		}, null, false, false);
	}
	
	public void Connect()
	{		
		statusText.text = "狀態: 正在與Rabboni建立連線...";
				
		subscribeChecked = false;
		prefsChecked = false;

		BluetoothLEHardwareInterface.ConnectToPeripheral (targetAddress, null, null, (address, serviceUUID, characteristicUUID) => {

			//Android / iOS return different UUID format, android give fullUUID, iOS give only UUID
			// testLog.text += System.String.Format("{0}x{1}\n", serviceUUID.Substring(0, 4), characteristicUUID.Substring(0, 4));

			if (!subscribeChecked)
			{
				if (console.IsEqual (serviceUUID, console.SubscribeServiceUUID) && console.IsEqual (characteristicUUID, console.SubscribeCharacteristic))
				{
					subscribeChecked = true;
				}
			}
			
			if (!prefsChecked)
			{
				if (console.IsEqual (serviceUUID, console.PrefsServiceUUID) && console.IsEqual (characteristicUUID, console.PrefsCharacteristic))
				{
					prefsChecked = true;
				}
			}
			
			
			if(!isConnected && subscribeChecked && prefsChecked)
			{
				console.moduleDic[address] = this;
				statusText.text = "狀態: 成功與Rabboni建立連線";
					
				targetAddressText.text = System.String.Format("目標裝置 MAC: {0}", address);
				isConnected = true;
								
				Invoke("ResetPrefs", 0.5f);				
				// Invoke("InitPrefs", InvokeDelay16G);
			}
			
		}, (disconnectedAddress) => {
			BluetoothLEHardwareInterface.Log ("Device disconnected: " + disconnectedAddress);
			if(console.moduleDic.ContainsKey(disconnectedAddress) )
			{
				console.moduleDic[disconnectedAddress].OnBLEDisconnected();
				console.moduleDic.Remove(disconnectedAddress);
			}
		});		
	}
	
	
	public void OnBLEDisconnected()
	{
		statusText.text = "狀態: 已中斷連線";
				
		isConnected = false;
				
		// lastAcc = Vector3.zero;
		// lastGyro = Vector3.zero;
					
		buttonSet.SetActive(true);
		buttonStopScan.SetActive(false);
	}
	
	
	
	public void ResetPrefs()
	{
		statusText.text = "狀態: 初始化Rabboni參數";
		string cmd = "4500020112010100000000001009C4";
		byte[] data = console.StringToByteArray(cmd);
		
		BluetoothLEHardwareInterface.WriteCharacteristic (targetAddress, console.PrefsServiceUUID, console.PrefsCharacteristic, data, data.Length, true, (characteristicUUID) => {

			statusText.text = "狀態: Rabboni初始化成功";
			
			Invoke("Subscribe", 0.5f);
		});
	}
	
	
	public void Subscribe()
	{
		statusText.text = "狀態: Rabboni連線中";
		
		BluetoothLEHardwareInterface.SubscribeCharacteristicWithDeviceAddress (targetAddress, console.SubscribeServiceUUID, console.SubscribeCharacteristic, null, (address, characteristicUUID, bytes) => {
						
			if(console.moduleDic.ContainsKey(address) )
			{
				console.moduleDic[address].OnSubscriveData(bytes);
			}
			
		});
	}
	
	public void OnSubscriveData(byte[] bytes)
	{
		string data = console.ByteArrayToString(bytes);
		
		// statusText.text = data;
		// statusText.text = System.String.Format("收到資料: {0}", data);
				
		int[] dataElement = new int[6];
		
		for(int i=0; i < 6; i++)
		{
			string tempHex = data.Substring(i*4, 4);
			short tempVal = System.Convert.ToInt16(tempHex, 16);
			dataElement[i] = tempVal;
		}
		
		//trueVal = 0-32767 / (32768/2g)
		
		lastAcc = new Vector3(dataElement[0], dataElement[1], dataElement[2])/16384f;
		
		//trueVal = 0-32767/ (32768/1000)

		lastGyro = new Vector3(dataElement[3], dataElement[4], dataElement[5])/32.768f;
		
		statusText.text = System.String.Format("lastAcc: {0}, lastGyro: {1}", lastAcc, lastGyro);
		
		/*if(lastAcc.magnitude > 1.5f || lastGyro.magnitude > 1000f)
		{
			
		}*/
		
	}
}
